-- RelicInspector.lua
-- Author: Thoralie

local addonName, addon = ...
local RelicSpells = addon.SpellIDByRelic

local DEBUG = 0

local options, optionsFrame, db

local modifierPressed = 0

local RelicTypeNames = {
	[1] = RELIC_SLOT_TYPE_BLOOD,
	[2] = RELIC_SLOT_TYPE_SHADOW,
	[3] = RELIC_SLOT_TYPE_IRON,
	[4] = RELIC_SLOT_TYPE_FROST,
	[5] = RELIC_SLOT_TYPE_FIRE,
	[6] = RELIC_SLOT_TYPE_FEL,
	[7] = RELIC_SLOT_TYPE_ARCANE,
	[8] = RELIC_SLOT_TYPE_LIFE,
	[9] = RELIC_SLOT_TYPE_WIND,
	[10] = RELIC_SLOT_TYPE_HOLY,
}

local RelicSlotsByArtifact = {
	[127829] = { 6, 2, 6}, -- Havoc DH
	[128832] = { 3, 7, 6}, -- Vengeance DH

	[128402] = { 1, 2, 3}, -- Blood DK
	[128292] = { 4, 2, 4}, -- Frost DK
	[128403] = { 5, 2, 1}, -- Unholy DK

	[128858] = { 7, 8, 7}, -- Balance Druid
	[128860] = { 4, 1, 8}, -- Feral Druid
	[128821] = { 5, 1, 8}, -- Guardian Druid
	[128306] = { 8, 4, 8}, -- Restoration Druid

	[128861] = { 9, 7, 3}, -- Beast Mastery Hunter
	[128826] = { 9, 1, 8}, -- Marksmanship Hunter
	[128808] = { 9, 3, 1}, -- Survival Hunter

	[127857] = { 7, 4, 7}, -- Arcane Mage
	[128820] = { 5, 7, 5}, -- Fire Mage
	[128862] = { 4, 7, 4}, -- Frost Mage

	[128938] = { 8, 9, 3}, -- Brewmaster Monk
	[128937] = { 4, 8, 9}, -- Mistweaver Monk
	[128940] = { 9, 3, 9}, -- Windwalker Monk

	[128823] = { 10, 8, 10}, -- Holy Paladin
	[128866] = { 10, 3, 7}, -- Protection Paladin
	[120978] = { 10, 5, 10}, -- Retribution Paladin

	[128868] = { 10, 2, 10}, -- Discipline Priest
	[128825] = { 10, 8, 10}, -- Holy Priest
	[128827] = { 2, 1, 2}, -- Shadow Priest

	[128870] = { 2, 3, 1}, -- Assassination Rogue
	[128872] = { 1, 3, 9}, -- Outlaw Rogue
	[128476] = { 6, 2, 6}, -- Subtlety Rogue

	[128935] = { 9, 4, 9}, -- Elemental Shaman
	[128819] = { 5, 3, 9}, -- Enhancement Shaman
	[128911] = { 8, 4, 8}, -- Restoration Shaman

	[128942] = { 2, 1, 2}, -- Affliction Warlock
	[128943] = { 2, 5, 6}, -- Demonology Warlock
	[128941] = { 6, 5, 6}, -- Destruction Warlock

	[128910] = { 3, 1, 2}, -- Arms Warrior
	[128908] = { 5, 9, 3}, -- Fury Warrior
	[128289] = { 3, 1, 5}, -- Protection Warrior
}

local hoverOptions = {
    [1] = "Never",
    [2] = "CTRL Key",
    [3] = "Always" 
}

local invHoverOptions = {
	["Never"] = 1,
	["CTRL Key"] = 2,
    ["Always"] = 3
}

local linkOptions = {
    [1] = "Never",
    [2] = "Always" 
}

local invLinkOptions = {
	["Never"] = 1,
    ["Always"] = 2
}

local defaults = {
	profile = {
		enabled = true,
		hoverTraitNames = "Always",
		hoverTraitDesc = "CTRL Key",
		linkTraitNames = "Always",
		linkTraitDesc = "Always",
	}
}

local function SetupOptions()
	if options then return options end

	options = {
		name = "RelicInspector",
		type = "group",
		args = {
			general = {
				name = "Options",
				type = "group",
				args = {
					enabled = {
						name = "Enable",
						desc = "Enables / Disables RelicInspector",
						type = "toggle",
						set = function(info,val) db.profile.enabled = val end,
						get = function(info) return db.profile.enabled end,
						order = 10
					},
					itemTooltipHeader = {
						name = "Item Tooltip Options",
						type = "header",
						order = 20
					},
					itemTooltipTraitNames = {
						name = "Show Trait Names:",
						desc = "When to show trait names on item tooltips",
						type = "select",
						width = "single",
						set = function(info,val) db.profile.hoverTraitNames = hoverOptions[val] end,
						get = function(info) return invHoverOptions[db.profile.hoverTraitNames] end,
						order = 30,
						values = hoverOptions
					},
					itemTooltipTraitDesc = {
						name = "Show Trait Descriptions:",
						desc = "When to show trait descriptions on item tooltips",
						type = "select",
						width = "single",
						set = function(info,val) db.profile.hoverTraitDesc = hoverOptions[val] end,
						get = function(info) return invHoverOptions[db.profile.hoverTraitDesc] end,
						order = 40,
						values = hoverOptions
					},
					linkTooltipHeader = {
						name = "Link Tooltip Options",
						type = "header",
						order = 50
					},
					linkTooltipTraitNames = {
						name = "Show Trait Names:",
						desc = "When to show trait names on link tooltips",
						type = "select",
						width = "single",
						set = function(info,val) db.profile.linkTraitNames = linkOptions[val] end,
						get = function(info) return invLinkOptions[db.profile.linkTraitNames] end,
						order = 60,
						values = linkOptions
					},
					linkTooltipTraitDesc = {
						name = "Show Trait Descriptions:",
						desc = "When to show trait descriptions on link tooltips",
						type = "select",
						width = "single",
						set = function(info,val) db.profile.linkTraitDesc = linkOptions[val] end,
						get = function(info) return invLinkOptions[db.profile.linkTraitDesc] end,
						order = 70,
						values = linkOptions
					},
				}
			},
		}
	}

	return options
end

local function initialize()
	db = _G.LibStub("AceDB-3.0"):New("RelicInspectorDB", defaults, "Default")
	local config = _G.LibStub("AceConfig-3.0")
	local dialog = _G.LibStub("AceConfigDialog-3.0")
	local options = SetupOptions()

	config:RegisterOptionsTable("RelicInspector", options)
	optionsFrame = dialog:AddToBlizOptions("RelicInspector", "RelicInspector", nil, "general")
end

local function DecorateTooltip(self)
	local _, link = self:GetItem()
	if type(link) == 'string' and db.profile.enabled == true then
		local _, itemID, _, relic1, relic2, relic3, _, _, _, _, _, upgradeID = strsplit(':', link)
		if upgradeID == '256' and nil ~= RelicSlotsByArtifact[tonumber(itemID)] then
			local relics = {relic1,relic2,relic3}
			for i = 1,3 do 
				if '' ~= relics[i] then
					local name, gemLink = GetItemGem(link,i)
					if type(gemLink) == 'string' then
						--Display the gem name, item level, and relic type
						local _, _, _, gemLevel = GetItemInfo(gemLink)
						local relicType = RelicTypeNames[RelicSlotsByArtifact[tonumber(itemID)][i]]

						if nil == relicType then
							relicType = "???"
						end
						self:AddDoubleLine(format('|cffffd400 (%s) %s|r', gemLevel or '???', gemLink or '???'), 
							format('|cff00ff00(%s)',relicType), 0, 1, 0, 0, 1, 0)

						-- Check options to see what else to display
						local showTraitNames, showTraitDesc
						if(self:GetName() == "ItemRefTooltip") then
							-- Item Links
							if invLinkOptions[db.profile.linkTraitNames] == 2 then
								showTraitNames = true
							else 
								showTraitNames = false
							end

							if invLinkOptions[db.profile.linkTraitDesc] == 2 then
								showTraitDesc = true
							else 
								showTraitDesc = false
							end
						else -- Game or Shopping Hover Tooltips
							if invHoverOptions[db.profile.hoverTraitNames] == 3 or
								(invHoverOptions[db.profile.hoverTraitNames] == 2 and modifierPressed ~= 0) then
								showTraitNames = true
							else 
								showTraitNames = false
							end

							if invHoverOptions[db.profile.hoverTraitDesc] == 3 or
								(invHoverOptions[db.profile.hoverTraitDesc] == 2 and modifierPressed ~= 0) then
								showTraitDesc = true
							else 
								showTraitDesc = false
							end
						end

						-- Let's add the Ranks and Trait names if we are supposed to
						if showTraitNames or showTraitDesc then
							local spellLookupKey = itemID .. '.' .. relics[i]
							local traitSpellID = RelicSpells[spellLookupKey]

							if nil ~= traitSpellID then
								--Add Trait Name
								if showTraitNames == true then
									local traitName = GetSpellInfo(traitSpellID)
									if nil ~= traitName then
										-- Assuming 1 Rank per relic right now. Will have to update this if relics start giving more than 1 rank
										self:AddLine(format(' ' .. RELIC_TOOLTIP_RANK_INCREASE, 1, traitName), 0, 1, 0, false)
									end
								end
								--Add Trait Description
								if showTraitDesc == true then
									local traitDesc = GetSpellDescription(traitSpellID)
									if nil ~= traitDesc then
										self:AddLine(format('|cffff78ff  %s|r', traitDesc), 1, 1, 0, true)
									end
								end
							end
						end
					end	
				end
			end
		end 
	end
end

GameTooltip:RegisterEvent("MODIFIER_STATE_CHANGED")
GameTooltip:HookScript("OnEvent", function(self, event, key, state)
	if event == "MODIFIER_STATE_CHANGED" and (key == "LCTRL" or key == "RCTRL") then
		modifierPressed = state
	end
end)

local f = CreateFrame('frame')
f:SetScript('OnEvent', function(self, event, arg1)
  if arg1 == addonName then
    self:UnregisterEvent('ADDON_LOADED')
    initialize()
  end
end)
f:RegisterEvent('ADDON_LOADED')

ItemRefTooltip:HookScript('OnTooltipSetItem', DecorateTooltip)
GameTooltip:HookScript('OnTooltipSetItem', DecorateTooltip)
ShoppingTooltip1:HookScript('OnTooltipSetItem', DecorateTooltip)
ShoppingTooltip2:HookScript('OnTooltipSetItem', DecorateTooltip)

